% Questo programma simula il decadimento numerico descritto su Eureka 46
% del 1986.

\datethis
\noinx

\italiano % sul mio sistema, significa "carica la sillabazione
          % italiana". Se sul vostro non funziona, cancellate la riga
          % o sostituitela opportunamente.

% In questo testo sono usati i seguenti caratteri, di codifica ascii
% maggiore di 127. Se il vostro sistema non li accetta, sostituiteli con
% le sequenze di escape indicate a fianco

%         \`a
%         \`e
%         \'e
%         \`{\i}
%         \`o
%         \`u

@* Decadimento numerico.
Questo programma simula il decadimento numerico descritto sul
fascicolo {\it Eureka} numero~46 uscito nel 1986. Probabilmente in
future versioni far anche altre cose; per ora si limita a leggere una
stringa e a farla decadere per un certo numero di volte.

Questa  la versione:

@d VERSION "1"

@ La struttura del programma  (per ora) molto semplice: si tratta di
leggere una stringa, chiamare per un po' di volte la funzione di
evoluzione, e di scrivere i risultati.
@c @<Inclusioni dei file di libreria@>;
@<Definizione della funzione |evoluzione()|@>;
@<Programma principale@>;

@ Viene incluso il file di libreria \.{stdio.h} per poter usare le
funzioni di input/output e il file \.{string.h} per le operazioni sulle
stringhe.

@<Incl...@>=
#include<stdio.h>
#include<string.h>

@ Viene definita la macro |LUN|, che rappresenta la lunghezza massima
delle stringhe gestite dal programma.
@d LUN 256

@ La funzione |evoluzione()| trasforma una stringa secondo le regole del
decadimento numerico. Se si trovano $n$ cifre uguali ad $a$, al loro posto
viene sostituita la stringa $na$. Se si trova una sola cifra $a$ (cio se la
cifra seguente  diversa), al suo posto viene sostituita la stringa
$1a$.

Questa funzione ha due parametri: |sin| (la stringa di input) e |sout|
(la stringa di output). Non ritorna alcun valore.

La variabile |l| contiene la lunghezza della stringa di input. La
variabile |i|  un indice che serve per scorrere la stringa di input
dall'inizio alla fine, analogamente la variabile |j| serve per scorrere la
stringa di output. La variabile |n|  un contatore che serve per
contare il numero di cifre uguali adiacenti.

@<Definizione della funzione |evoluzione()|@>=
void evoluzione(char sin[], char sout[])
{
        char buffer[LUN];

        int i,j,l,n;

        l=strlen(sin);
        i=j=0;
        n=1;
        while (i<l) {

                if ( sin[i]==sin[i+1] ) {
                        n++; /* aumenta il contatore */
                }
                else {
                        sprintf(sout+j,"%d",n); /* copia |n| sulla stringa */
                        j++;
                        sout[j]=sin[i]; /* copia la cifra sulla stringa */
                        j++;
                        n=1; /* reinizializza il contatore */
                }
                i++;
        }
        sout[j]='\0'; /* inserisco il terminatore alla fine della stringa |sout| */
}

@ Il programma principale legge da tastiera una stringa, chiama per
alcune volte la funzione |evoluzione()| (si ferma quando la stringa 
pi lunga di una riga dello schermo), poi termina.

@<Progr...@>=
void main(void)
{
        char stringa[LUN];
        char prova[LUN];
        printf("This is numdec, Version %s\n",VERSION);
        printf("\nDammi la stringa: ");
        scanf("%s",stringa);

        puts(stringa);
        while (strlen(stringa)<80) {
                evoluzione(stringa,prova);
                puts(prova);
                strcpy(stringa,prova);
        }
}
